/*
* Copyright (C) Mellanox Technologies Ltd, 2001-2018. ALL RIGHTS RESERVED.
*
* See file LICENSE for terms.
*/

#ifndef NVME_MLX5_IFC_H
#define NVME_MLX5_IFC_H

#include "mlx5_ifc.h"

enum {
    MLX5_CMD_OPCODE_SEND_QP_NVMF_CC                = 0xb01,
    MLX5_CMD_OPCODE_QUERY_NVMF_CC_RESPONSE         = 0xb02,
    MLX5_CMD_OPCODE_QUERY_EMULATED_FUNCTIONS_INFO  = 0xb03,
};

enum {
	MLX5_QPC_OFFLOAD_TYPE_NVMX         = 0x3,
};

enum {
	MLX5_NVME_SQ_OFFLOAD_TYPE_SQE = 0x0,
	MLX5_NVME_SQ_OFFLOAD_TYPE_DOORBELL_ONLY = 0x1,
	MLX5_NVME_SQ_OFFLOAD_TYPE_NVMF_CC = 0x2,
};

enum {
    MLX5_NVME_SQ_STATE_INIT   = 0x0,
    MLX5_NVME_SQ_STATE_RDY    = 0x1,
    MLX5_NVME_SQ_STATE_ERROR  = 0x2,
};

enum {
    MLX5_NVME_SQ_FIELD_SELECT_QPN   = 1 << 0,
    MLX5_NVME_SQ_FIELD_SELECT_STATE = 1 << 1,
};

enum {
	MLX5_NVME_CQ_OFFLOAD_TYPE_CQE = 0x0,
	MLX5_NVME_CQ_OFFLOAD_TYPE_NVMF_CQE_CC = 0x2
};

struct mlx5_ifc_nvme_namespace_bits {
	uint8_t    src_nsid[0x20];

	uint8_t    dst_nsid[0x20];

	uint8_t    reserved_at_40[0x8];
	uint8_t    lba_size[0x8];
	uint8_t    metadata_size[0x10];

	uint8_t    reserved_at_60[0x20];
};

struct mlx5_ifc_nvme_cq_bits {
    uint8_t    modify_field_select[0x40];

    uint8_t    reserved_at_40[0x10];
    uint8_t    function_id[0x10];

    uint8_t    reserved_at_60[0x20];

    uint8_t    reserved_at_80[0x4];
    uint8_t    offload_type[0x4];
    uint8_t    reserved_at_88[0x18];

    uint8_t    reserved_at_a0[0x18];
    uint8_t    msix_vector[0x8];

    uint8_t    nvme_base_addr[0x40];

    uint8_t    reserved_at_100[0x8];
    uint8_t    nvme_log_entry_size[0x8];
    uint8_t    nvme_num_of_entries[0x10];

    uint8_t    nvme_doorbell_offset[0x20];

    uint8_t    cq_period_mode[0x1];
    uint8_t    reserved_at_144[0x3];
    uint8_t    cq_period[0xc];
    uint8_t    cq_max_count[0x10];

    uint8_t    reserved_at_160[0x6a0];
};

struct mlx5_ifc_nvme_sq_bits {
    uint8_t    modify_field_select[0x40];

    uint8_t    reserved_at_40[0x10];
    uint8_t    vhca_id[0x10];

    uint8_t    reserved_at_60[0x20];

    uint8_t    state[0x4];
    uint8_t    offload_type[0x4];
    uint8_t    qpn[0x18];

    uint8_t    nvme_cq_id[0x10];
    uint8_t    reserved_at_b0[0x3];
    uint8_t    log_nvme_page_size[0x5];
    uint8_t    msix_vector[0x8];

    uint8_t    nvme_base_addr[0x40];

    uint8_t    max_transaction_size[0x8];
    uint8_t    nvme_log_entry_size[0x8];
    uint8_t    nvme_num_of_entries[0x10];

    uint8_t    nvme_doorbell_offset[0x20];

    uint8_t    mkey_buffer_umem_offset[0x40];

    uint8_t    mkey_buffer_umem_id[0x20];

    uint8_t    reserved_at_1a0[0x640];

    uint8_t    reserved_at_7e0[0x10];
    uint8_t    num_of_namespaces[0x10];

    struct mlx5_ifc_nvme_namespace_bits nvme_namespace[0];
};

struct mlx5_ifc_query_nvmf_cc_in_bits {
    uint8_t    opcode[0x10];
    uint8_t    uid[0x10];

    uint8_t    reserved_at_20[0x10];
    uint8_t    op_mod[0x10];

    uint8_t    nvme_sq[0x20];

    uint8_t    reserved_at_60[0x1];
    uint8_t    cmd_on_behalf[0x1];
    uint8_t    reserved_at_62[0xe];
    uint8_t    vhca_id[0x10];
};

struct mlx5_ifc_query_nvmf_cc_out_bits {
    uint8_t    status[0x8];
    uint8_t    reserved_at_8[0x18];

    uint8_t    syndrome[0x20];

    uint8_t    reserved_at_40[0x3c];
    uint8_t    cc_response_status[0x4];

    uint8_t    cc_response[0x80];

};

struct mlx5_ifc_send_qp_nvmf_cc_in_bits {
    uint8_t    opcode[0x10];
    uint8_t    uid[0x10];

    uint8_t    reserved_at_20[0x10];
    uint8_t    op_mod[0x10];

    uint8_t    nvme_sq[0x20];

    uint8_t    reserved_at_60[0x1];
    uint8_t    cmd_on_behalf[0x1];
    uint8_t    reserved_at_62[0xe];
    uint8_t    vhca_id[0x10];

    uint8_t    reserved_at_80[0x10];
    uint8_t    ext_data_length[0x10];

    uint8_t    ext_data_umem_id[0x20];

    uint8_t    ext_data_umem_offset[0x40];

    uint8_t    reserved_at_100[0x100];

    uint8_t    nvmf_cc[16][0x20];
};

struct mlx5_ifc_send_qp_nvmf_cc_out_bits {
    uint8_t    status[0x8];
    uint8_t    reserved_at_8[0x18];

    uint8_t    syndrome[0x20];

    uint8_t    reserved_at_40[0x8];
    uint8_t    qpn[0x18];

    uint8_t    reserved_at_60[0x20];
};

enum {
        MLX5_OBJ_TYPE_NVME_QP = 0x0007,
        MLX5_OBJ_TYPE_NVME_CQ = 0x0009
};

struct mlx5_ifc_nvme_device_emulation_object_bits {
    uint8_t    modify_field_select[0x40];

    uint8_t    reserved_at_40[0x10];
    uint8_t    vhca_id[0x10];

    uint8_t    enabled[0x1];
    uint8_t    reserved_at_61[0x1f];

    uint8_t    register_data[0][0x20];
};

struct mlx5_ifc_nvme_create_qp_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         reserved_at_40[0x8];
    uint8_t         input_qpn[0x18];

    uint8_t         reserved_at_60[0x1];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         reserved_at_62[0xe];
    uint8_t         vhca_id[0x10];

    uint8_t         opt_param_mask[0x20];

    uint8_t         reserved_at_a0[0x20];

    struct          mlx5_ifc_qpc_bits qpc;

    uint8_t         wq_umem_offset[0x40];

    uint8_t         wq_umem_id[0x20];

    uint8_t         reserved_at_860[0x20];

    uint8_t         pas[0][0x40];
};

struct mlx5_ifc_nvme_cmd_hca_cap_bits {
    uint8_t         reserved_at_0[0x30];
    uint8_t         vhca_id[0x10];

    uint8_t         reserved_at_40[0x40];

    uint8_t         log_max_srq_sz[0x8];
    uint8_t         log_max_qp_sz[0x8];
    uint8_t         reserved_at_90[0xb];
    uint8_t         log_max_qp[0x5];

    uint8_t         reserved_at_a0[0xb];
    uint8_t         log_max_srq[0x5];
    uint8_t         reserved_at_b0[0x10];

    uint8_t         reserved_at_c0[0x8];
    uint8_t         log_max_cq_sz[0x8];
    uint8_t         reserved_at_d0[0xb];
    uint8_t         log_max_cq[0x5];

    uint8_t         log_max_eq_sz[0x8];
    uint8_t         reserved_at_e8[0x2];
    uint8_t         log_max_mkey[0x6];
    uint8_t         reserved_at_f0[0x4];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         device_emulation_manager[0x1];
    uint8_t         reserved_at_f6[0x6];
    uint8_t         log_max_eq[0x4];

    uint8_t         max_indirection[0x8];
    uint8_t         fixed_buffer_size[0x1];
    uint8_t         log_max_mrw_sz[0x7];
    uint8_t         force_teardown[0x1];
    uint8_t         reserved_at_111[0x1];
    uint8_t         log_max_bsf_list_size[0x6];
    uint8_t         umr_extended_translation_offset[0x1];
    uint8_t         null_mkey[0x1];
    uint8_t         log_max_klm_list_size[0x6];

    uint8_t         reserved_at_120[0xa];
    uint8_t         log_max_ra_req_dc[0x6];
    uint8_t         reserved_at_130[0xa];
    uint8_t         log_max_ra_res_dc[0x6];

    uint8_t         reserved_at_140[0xa];
    uint8_t         log_max_ra_req_qp[0x6];
    uint8_t         reserved_at_150[0xa];
    uint8_t         log_max_ra_res_qp[0x6];

    uint8_t         end_pad[0x1];
    uint8_t         cc_query_allowed[0x1];
    uint8_t         cc_modify_allowed[0x1];
    uint8_t         start_pad[0x1];
    uint8_t         cache_line_128byte[0x1];
    uint8_t         reserved_at_165[0xa];
    uint8_t         qcam_reg[0x1];
    uint8_t         gid_table_size[0x10];

    uint8_t         out_of_seq_cnt[0x1];
    uint8_t         vport_counters[0x1];
    uint8_t         retransmission_q_counters[0x1];
    uint8_t         debug[0x1];
    uint8_t         modify_rq_counter_set_id[0x1];
    uint8_t         rq_delay_drop[0x1];
    uint8_t         max_qp_cnt[0xa];
    uint8_t         pkey_table_size[0x10];

    uint8_t         vport_group_manager[0x1];
    uint8_t         vhca_group_manager[0x1];
    uint8_t         ib_virt[0x1];
    uint8_t         eth_virt[0x1];
    uint8_t         vnic_env_queue_counters[0x1];
    uint8_t         ets[0x1];
    uint8_t         nic_flow_table[0x1];
    uint8_t         eswitch_flow_table[0x1];
    uint8_t         device_memory[0x1];
    uint8_t         mcam_reg[0x1];
    uint8_t         pcam_reg[0x1];
    uint8_t         local_ca_ack_delay[0x5];
    uint8_t         port_module_event[0x1];
    uint8_t         enhanced_error_q_counters[0x1];
    uint8_t         ports_check[0x1];
    uint8_t         reserved_at_1b3[0x1];
    uint8_t         disable_link_up[0x1];
    uint8_t         beacon_led[0x1];
    uint8_t         port_type[0x2];
    uint8_t         num_ports[0x8];

    uint8_t         reserved_at_1c0[0x1];
    uint8_t         pps[0x1];
    uint8_t         pps_modify[0x1];
    uint8_t         log_max_msg[0x5];
    uint8_t         reserved_at_1c8[0x4];
    uint8_t         max_tc[0x4];
    uint8_t         reserved_at_1d0[0x1];
    uint8_t         dcbx[0x1];
    uint8_t         general_notification_event[0x1];
    uint8_t         reserved_at_1d3[0x2];
    uint8_t         fpga[0x1];
    uint8_t         rol_s[0x1];
    uint8_t         rol_g[0x1];
    uint8_t         reserved_at_1d8[0x1];
    uint8_t         wol_s[0x1];
    uint8_t         wol_g[0x1];
    uint8_t         wol_a[0x1];
    uint8_t         wol_b[0x1];
    uint8_t         wol_m[0x1];
    uint8_t         wol_u[0x1];
    uint8_t         wol_p[0x1];

    uint8_t         stat_rate_support[0x10];
    uint8_t         reserved_at_1f0[0xc];
    uint8_t         cqe_version[0x4];

    uint8_t         compact_address_vector[0x1];
    uint8_t         striding_rq[0x1];
    uint8_t         reserved_at_202[0x1];
    uint8_t         ipoib_enhanced_offloads[0x1];
    uint8_t         ipoib_basic_offloads[0x1];
    uint8_t         reserved_at_205[0x1];
    uint8_t         repeated_block_disabled[0x1];
    uint8_t         umr_modify_entity_size_disabled[0x1];
    uint8_t         umr_modify_atomic_disabled[0x1];
    uint8_t         umr_indirect_mkey_disabled[0x1];
    uint8_t         umr_fence[0x2];
    uint8_t         reserved_at_20c[0x3];
    uint8_t         drain_sigerr[0x1];
    uint8_t         cmdif_checksum[0x2];
    uint8_t         sigerr_cqe[0x1];
    uint8_t         reserved_at_213[0x1];
    uint8_t         wq_signature[0x1];
    uint8_t         sctr_data_cqe[0x1];
    uint8_t         bsf_in_create_mkey[0x1];
    uint8_t         sho[0x1];
    uint8_t         tph[0x1];
    uint8_t         rf[0x1];
    uint8_t         dct[0x1];
    uint8_t         qos[0x1];
    uint8_t         eth_net_offloads[0x1];
    uint8_t         roce[0x1];
    uint8_t         atomic[0x1];
    uint8_t         reserved_at_21f[0x1];

    uint8_t         cq_oi[0x1];
    uint8_t         cq_resize[0x1];
    uint8_t         cq_moderation[0x1];
    uint8_t         reserved_at_223[0x3];
    uint8_t         cq_eq_remap[0x1];
    uint8_t         pg[0x1];
    uint8_t         block_lb_mc[0x1];
    uint8_t         reserved_at_229[0x1];
    uint8_t         scqe_break_moderation[0x1];
    uint8_t         cq_period_start_from_cqe[0x1];
    uint8_t         cd[0x1];
    uint8_t         reserved_at_22d[0x1];
    uint8_t         apm[0x1];
    uint8_t         vector_calc[0x1];
    uint8_t         umr_ptr_rlky[0x1];
    uint8_t         imaicl[0x1];
    uint8_t         reserved_at_232[0x4];
    uint8_t         qkv[0x1];
    uint8_t         pkv[0x1];
    uint8_t         set_deth_sqpn[0x1];
    uint8_t         reserved_at_239[0x3];
    uint8_t         xrc[0x1];
    uint8_t         ud[0x1];
    uint8_t         uc[0x1];
    uint8_t         rc[0x1];

    uint8_t         uar_4k[0x1];
    uint8_t         reserved_at_241[0x9];
    uint8_t         uar_sz[0x6];
    uint8_t         reserved_at_250[0x8];
    uint8_t         log_pg_sz[0x8];

    uint8_t         bf[0x1];
    uint8_t         driver_version[0x1];
    uint8_t         pad_tx_eth_packet[0x1];
    uint8_t         reserved_at_263[0x2];
    uint8_t         qp_by_name[0x1];
    uint8_t         reserved_at_266[0x5];
    uint8_t         log_bf_reg_size[0x5];

    uint8_t         reserved_at_270[0xb];
    uint8_t         lag_master[0x1];
    uint8_t         num_lag_ports[0x4];

    uint8_t         reserved_at_280[0x10];
    uint8_t         max_wqe_sz_sq[0x10];

    uint8_t         reserved_at_2a0[0x10];
    uint8_t         max_wqe_sz_rq[0x10];

    uint8_t         max_flow_counter_31_16[0x10];
    uint8_t         max_wqe_sz_sq_dc[0x10];

    uint8_t         reserved_at_2e0[0x7];
    uint8_t         max_qp_mcg[0x19];

    uint8_t         reserved_at_300[0x18];
    uint8_t         log_max_mcg[0x8];

    uint8_t         reserved_at_320[0x3];
    uint8_t         log_max_transport_domain[0x5];
    uint8_t         reserved_at_328[0x3];
    uint8_t         log_max_pd[0x5];
    uint8_t         reserved_at_330[0xb];
    uint8_t         log_max_xrcd[0x5];

    uint8_t         nic_receive_steering_discard[0x1];
    uint8_t         receive_discard_vport_down[0x1];
    uint8_t         transmit_discard_vport_down[0x1];
    uint8_t         reserved_at_343[0x5];
    uint8_t         log_max_flow_counter_bulk[0x8];
    uint8_t         max_flow_counter_15_0[0x10];


    uint8_t         reserved_at_360[0x3];
    uint8_t         log_max_rq[0x5];
    uint8_t         reserved_at_368[0x3];
    uint8_t         log_max_sq[0x5];
    uint8_t         reserved_at_370[0x3];
    uint8_t         log_max_tir[0x5];
    uint8_t         reserved_at_378[0x3];
    uint8_t         log_max_tis[0x5];

    uint8_t         basic_cyclic_rcv_wqe[0x1];
    uint8_t         reserved_at_381[0x2];
    uint8_t         log_max_rmp[0x5];
    uint8_t         reserved_at_388[0x3];
    uint8_t         log_max_rqt[0x5];
    uint8_t         reserved_at_390[0x3];
    uint8_t         log_max_rqt_size[0x5];
    uint8_t         reserved_at_398[0x3];
    uint8_t         log_max_tis_per_sq[0x5];

    uint8_t         ext_stride_num_range[0x1];
    uint8_t         reserved_at_3a1[0x2];
    uint8_t         log_max_stride_sz_rq[0x5];
    uint8_t         reserved_at_3a8[0x3];
    uint8_t         log_min_stride_sz_rq[0x5];
    uint8_t         reserved_at_3b0[0x3];
    uint8_t         log_max_stride_sz_sq[0x5];
    uint8_t         reserved_at_3b8[0x3];
    uint8_t         log_min_stride_sz_sq[0x5];

    uint8_t         hairpin[0x1];
    uint8_t         reserved_at_3c1[0x2];
    uint8_t         log_max_hairpin_queues[0x5];
    uint8_t         reserved_at_3c8[0x3];
    uint8_t         log_max_hairpin_wq_data_sz[0x5];
    uint8_t         reserved_at_3d0[0x3];
    uint8_t         log_max_hairpin_num_packets[0x5];
    uint8_t         reserved_at_3d8[0x3];
    uint8_t         log_max_wq_sz[0x5];

    uint8_t         nic_vport_change_event[0x1];
    uint8_t         disable_local_lb_uc[0x1];
    uint8_t         disable_local_lb_mc[0x1];
    uint8_t         log_min_hairpin_wq_data_sz[0x5];
    uint8_t         reserved_at_3e8[0x3];
    uint8_t         log_max_vlan_list[0x5];
    uint8_t         reserved_at_3f0[0x3];
    uint8_t         log_max_current_mc_list[0x5];
    uint8_t         reserved_at_3f8[0x3];
    uint8_t         log_max_current_uc_list[0x5];

    uint8_t         general_obj_types[0x40];

    uint8_t         reserved_at_440[0x40];

    uint8_t         reserved_at_480[0x3];
    uint8_t         log_max_l2_table[0x5];
    uint8_t         reserved_at_488[0x8];
    uint8_t         log_uar_page_sz[0x10];

    uint8_t         reserved_at_4a0[0x20];
    uint8_t         device_frequency_mhz[0x20];
    uint8_t         device_frequency_khz[0x20];

    uint8_t         reserved_at_500[0x20];
    uint8_t         num_of_uars_per_page[0x20];
    uint8_t         reserved_at_540[0x40];

    uint8_t         reserved_at_580[0x3d];
    uint8_t         cqe_128_always[0x1];
    uint8_t         cqe_compression_128[0x1];
    uint8_t         cqe_compression[0x1];

    uint8_t         cqe_compression_timeout[0x10];
    uint8_t         cqe_compression_max_num[0x10];

    uint8_t         reserved_at_5e0[0x10];
    uint8_t         tag_matching[0x1];
    uint8_t         rndv_offload_rc[0x1];
    uint8_t         rndv_offload_dc[0x1];
    uint8_t         log_tag_matching_list_sz[0x5];
    uint8_t         reserved_at_5f8[0x3];
    uint8_t         log_max_xrq[0x5];

    uint8_t         affiliate_nic_vport_criteria[0x8];
    uint8_t         native_port_num[0x8];
    uint8_t         num_vhca_ports[0x8];
    uint8_t         reserved_at_618[0x6];
    uint8_t         sw_owner_id[0x1];
    uint8_t         reserved_at_61f[0x1e1];
};

struct mlx5_ifc_nvme_emulation_cap_bits {
    uint8_t    nvme_offload_type_sqe[0x1];
    uint8_t    nvme_offload_type_doorbell_only[0x1];
    uint8_t    nvme_offload_type_command_capsule[0x1];
    uint8_t    log_max_nvme_offload_namespaces[0x5];
    uint8_t    reserved_at_7[0x10];
    uint8_t    total_emulated_pfs[0x8];

    uint8_t    reserved_at_20[0x10];
    uint8_t    registers_size[0x10];

    uint8_t    reserved_at_40[0x13];
    uint8_t    log_max_emulated_sq[0x5];
    uint8_t    reserved_at_58[0x3];
    uint8_t    log_max_emulated_cq[0x5];

    uint8_t    reserved_at_60[0x7a0];
};

struct mlx5_ifc_nvme_query_emulated_functions_info_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];

    uint8_t         op_mod[0x10];
    uint8_t         reserved_at_40[0x40];
};

struct mlx5_ifc_nvme_emulated_pf_info_bits {
    uint8_t         pf_pci_number[0x10];
    uint8_t         pf_vhca_id[0x10];

    uint8_t         num_of_vfs[0x10];
    uint8_t         vfs_base_vhca_id[0x10];
};

struct mlx5_ifc_nvme_query_emulated_functions_info_out_bits {
    uint8_t    status[0x8];
    uint8_t    reserved_at_8[0x18];

    uint8_t    syndrome[0x20];
    uint8_t    reserved_at_46[0x38];

    uint8_t    num_emulated_pfs[0x8];
    struct     mlx5_ifc_nvme_emulated_pf_info_bits emulated_pf_info[0];
};

struct mlx5_ifc_nvme_mkc_bits {
    uint8_t         reserved_at_0[0x1];
    uint8_t         free[0x1];
    uint8_t         reserved_at_2[0x1];
    uint8_t         access_mode_4_2[0x3];
    uint8_t         reserved_at_6[0x7];
    uint8_t         relaxed_ordering_write[0x1];
    uint8_t         reserved_at_e[0x1];
    uint8_t         small_fence_on_rdma_read_response[0x1];
    uint8_t         umr_en[0x1];
    uint8_t         a[0x1];
    uint8_t         rw[0x1];
    uint8_t         rr[0x1];
    uint8_t         lw[0x1];
    uint8_t         lr[0x1];
    uint8_t         access_mode_1_0[0x2];
    uint8_t         reserved_at_18[0x8];

    uint8_t         qpn[0x18];
    uint8_t         mkey_7_0[0x8];

    uint8_t         reserved_at_40[0x20];

    uint8_t         length64[0x1];
    uint8_t         bsf_en[0x1];
    uint8_t         sync_umr[0x1];
    uint8_t         reserved_at_63[0x2];
    uint8_t         expected_sigerr_count[0x1];
    uint8_t         reserved_at_66[0x1];
    uint8_t         en_rinval[0x1];
    uint8_t         pd[0x18];

    uint8_t         start_addr[0x40];

    uint8_t         len[0x40];

    uint8_t         bsf_octword_size[0x20];

    uint8_t         reserved_at_120[0x80];

    uint8_t         translations_octword_size[0x20];

    uint8_t         reserved_at_1c0[0x1b];
    uint8_t         log_entity_size[0x5];

    uint8_t         reserved_at_1e0[0x20];
};

struct mlx5_ifc_nvme_create_mkey_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         reserved_at_40[0x20];

    uint8_t         pg_access[0x1];
    uint8_t         mkey_umem_valid[0x1];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         reserved_at_63[0xd];
    uint8_t         vhca_id[0x10];

    struct          mlx5_ifc_nvme_mkc_bits memory_key_mkey_entry;

    uint8_t         reserved_at_280[0x80];

    uint8_t         translations_octword_actual_size[0x20];

    uint8_t         mkey_umem_id[0x20];

    uint8_t         mkey_umem_offset[0x40];

    uint8_t         reserved_at_380[0x10];
    uint8_t         bsf_octword_actual_size[0x10];

    uint8_t         reserved_at_3a0[0x4e0];

    uint8_t         klm_pas_mtt_bsf[0][0x20];
};

struct mlx5_ifc_nvme_create_psv_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         vhca_id[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         num_psv[0x4];
    uint8_t         reserved_at_44[0x3];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         pd[0x18];

    uint8_t         reserved_at_60[0x20];
};

struct mlx5_ifc_nvme_init_hca_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         reserved_at_40[0x20];

    uint8_t         m_pci_port_child[0x1];
    uint8_t         m_vhca_mk_child[0x1];

    uint8_t         reserved_at_42[0xe];
    uint8_t         parent_vhca_id[0x10];

    uint8_t         sw_owner_id[4][0x20];
};

struct mlx5_ifc_nvme_rst2init_qp_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         reserved_at_40[0x8];
    uint8_t         qpn[0x18];

    uint8_t         reserved_at_60[0x1];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         reserved_at_62[0xe];
    uint8_t         vhca_id[0x10];

    uint8_t         opt_param_mask[0x20];

    uint8_t         reserved_at_a0[0x20];

    struct          mlx5_ifc_qpc_bits qpc;

    uint8_t         reserved_at_800[0x80];
};

struct mlx5_ifc_nvme_init2rtr_qp_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         reserved_at_40[0x8];
    uint8_t         qpn[0x18];

    uint8_t         reserved_at_60[0x1];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         reserved_at_62[0xe];
    uint8_t         vhca_id[0x10];

    uint8_t         opt_param_mask[0x20];

    uint8_t         reserved_at_a0[0x20];

    struct          mlx5_ifc_qpc_bits qpc;

    uint8_t         reserved_at_800[0x80];
};

struct mlx5_ifc_nvme_rtr2rts_qp_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         reserved_at_40[0x8];
    uint8_t         qpn[0x18];

    uint8_t         reserved_at_60[0x1];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         reserved_at_62[0xe];
    uint8_t         vhca_id[0x10];

    uint8_t         opt_param_mask[0x20];

    uint8_t         reserved_at_a0[0x20];

    struct          mlx5_ifc_qpc_bits qpc;

    uint8_t         reserved_at_800[0x80];
};

struct mlx5_ifc_nvme_rts2rts_qp_in_bits {
    uint8_t         opcode[0x10];
    uint8_t         uid[0x10];

    uint8_t         reserved_at_20[0x10];
    uint8_t         op_mod[0x10];

    uint8_t         reserved_at_40[0x8];
    uint8_t         qpn[0x18];

    uint8_t         reserved_at_60[0x1];
    uint8_t         cmd_on_behalf[0x1];
    uint8_t         reserved_at_62[0xe];
    uint8_t         vhca_id[0x10];

    uint8_t         opt_param_mask[0x20];

    uint8_t         reserved_at_a0[0x20];

    struct          mlx5_ifc_qpc_bits qpc;

    uint8_t         reserved_at_800[0x80];
};

#endif /* NVME_MLX5_IFC_H */
